/**
 *
 * @File : nsSysteme.cxx
 *
 * 2011
 * @Synopsis : definition des wrappers non inline des fonctions syst
 *
**/

#include <string>
#include <fcntl.h>       // O_CREAT, open()
#include <sys/types.h>   // mode_t, ssize_t
#include <unistd.h> // read, write
#include <cstddef>       // size_t
#include "../Include/CException.hpp"
#include "../Include/nsSysteme.h"

using namespace std;     // string
using namespace nsUtil;     // CException

int nsSysteme::Open (const char * pathname, int flags, ::mode_t mode)
{
    int Res;
    if (!(flags & O_CREAT))
        throw CException ("Open()" + string (" fichier :") + pathname +
                       ". Un parametre de trop", CstOpenArgPlus);
    if (-1 == (Res = ::open (pathname, flags, mode)))
        throw CException ("open() " + string (" fichier :") + pathname);

    return Res;

} // Open()

int nsSysteme::Open (const char * pathname, int flags)
{
    int Res;
    if (flags & O_CREAT)
        throw CException ("Open()" + string (" fichier :") + pathname +
                    ". Il manque un parametre", CstOpenArgPlus);
    if (-1 == (Res = ::open (pathname, flags)))
        throw CException ("open() " + string (" fichier :") + pathname);

    return Res;

} // Open()

//Write
ssize_t nsSysteme::Write(int fd, const void *buf, size_t count){
    if (count >= SIZE_MAX)
        throw CException ("Erreur");
    ssize_t res;
    if (-1 == (res = ::write (fd, buf, count)))
        throw (CException("Write"));
    return res;
}

//Read
ssize_t nsSysteme::Read   (int fd, void * buf, size_t count)
{
    if (count == 0)
        throw CException ("Erreur");
    ssize_t res;
    if (-1 == (res = ::read (fd, buf, count)))
        throw (CException("Read"));
    return res;
}

//close ()
void  nsSysteme::Close  (int fd){
    if (-1 == ::close (fd))
        throw (CException("Close"));
}

sighandler_t nsSysteme::Signal(int signum,
                       sighandler_t handler)
{
    struct sigaction action, oldaction;
    action.sa_handler = handler;
    if (sigaction (signum, &action, &oldaction))
        throw (CException("signal"));
    return oldaction.sa_handler;
}

int nsSysteme::Sigaction(int signum, const struct sigaction * act,
                         struct sigaction * oldact){
    int res;
    if (0 != (res = sigaction (signum, act, oldact)))
        throw (CException("Sigaction"));
    return res;
}

int nsSysteme::Sigemptyset (sigset_t *set) {
    int res;
    if (0 != (res = sigemptyset(set)))
        throw (CException("Sigemptyset"));
    return res;
}

int nsSysteme::Sigaddset (sigset_t *set, int signo){
    int res;
    if (0 != (res = sigaddset (set, signo)))
        throw (CException("Sigaddset"));
    return res;
}

pid_t nsSysteme::Fork()
{
    pid_t res;
    if (-1 == (res = fork()))
        throw (CException("Fork ()"));
    return res;
}



int nsSysteme::System(const char *command)
{
    if (nullptr == command)
        throw (CException("System ()"));
    int res;
    if (-1 == (res = system(command)))
        throw (CException("System ()"));
    return res;
}

int nsSysteme::Kill(pid_t pid, int sig)
{
    int res;
    if (-1 == (res = kill(pid, sig)))
        throw (CException("Kill ()"));
    return res;
}
